#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

from xoeuf.odoo.fields import Field as Base


class Property(Base):
    '''A property-like field.

    This is always non-store field.  In fact, you may only pass the getter,
    setter and deleter functions.

    .. warning:: This is a kind of hack.

       Normal Python `properties <property>`:class: are not copied through
       Odoo inheritance mechanisms.  This `Property` is a Field and Odoo
       accounts for it.  But, it is NOT an actual ORM field in the sense that
       it never interacts with the DB.  Therefore:

       - You MUST NEVER use ``write()`` or ``create()`` on these type of
         fields.  You may update the value via the `setter` (i.e assign it).

       - You cannot ``search()`` for this type of fields.

       - You cannot use Property fields in views.

         The main difference to any other computed field is that Property is
         untyped.  We don't enforce any type check on the value returned by
         getters.  But, then, we cannot communicate which widget can manage
         the value of a Property.

    The `getter`, `setter` and `deleter` functions receive (and thus we
    required it) a singleton recordset instance.  The `onsetup` function
    receive the field instance, and the model on which the field is being
    setup.

    Usage::

       @Property
       def result(self):
           return 1

       @result.setter
       def result(self, value):
          pass

    You may also do::

       def _set_result(self, value):
           pass

       result = Property(lambda s: 1, setter=_set_result)
       del _set_result

    '''
    # This is the best of the three major versions of Odoo we support.  This
    # create a __slot__ and avoids that these values go to the
    # __getattr__/__setattr__ mechanism in fields.Field.  But it also, means
    # that we need to do some tricks in the `__init__`:meth: to make all
    # versions happy.
    _slots = {
        'property_getter': None,
        'property_setter': None,
        'property_deleter': None,
        'property_onsetup': None,
    }
    type = 'python-property'  # needed to satisfy ir.models.field

    def __init__(self, getter, setter=None, deleter=None, onsetup=None,
                 **kwargs):
        # Notice we don't abide by the expected fields signature.  Instead, we
        # require one that is compatible with `property`; but we ensure that
        # Odoo sees this Property as normal field with custom attributes.  We
        # allow for inheritance (_inherits) of this field.  Such inheritance
        # is done via a related field, but otherwise custom `write/create`
        # that treat properties could be ignored.
        from xoutil.symbols import Unset
        kw = {
            'inherited': kwargs.get('inherited', None),
            'related': kwargs.get('related', None),
            'related_sudo': kwargs.get('related_sudo', False),
        }
        super(Property, self).__init__(
            # Odoo ignores arguments which are None, therefore, let's force
            # them with Unset.  Odoo 9 and 10, uses this args to call
            # field.new(**args), and also to restore the values of the _slots
            # to provided values instead of default ones.  This last thing is
            # done when building the specific field for each Model Class (see
            # our `registry.rst`); cf. Field._setup_attrs.
            #
            # WARNING: If you remove this thinking that put those values just
            # a few lines below, you're fooling yourself.
            property_getter=getter or Unset,
            property_setter=setter or Unset,
            property_deleter=deleter or Unset,
            property_onsetup=onsetup or Unset,

            compute=getter,
            store=False,
            copy=False,
            **kw
        )
        # In any case, we need those values now to make `new()` below work.
        self.property_getter = getter or Unset
        self.property_setter = setter or Unset
        self.property_deleter = deleter or Unset
        self.property_onsetup = onsetup or Unset

    def new(self, **kwargs):
        # Ensure the property getter, setter and deleter are provided.  This
        # is used in `setter`:meth: and `deleter`:meth:.
        setter = kwargs.pop('setter', self.property_setter)
        deleter = kwargs.pop('deleter', self.property_deleter)
        onsetup = kwargs.pop('onsetup', self.property_onsetup)
        return type(self)(
            self.property_getter,
            setter=setter,
            deleter=deleter,
            onsetup=onsetup,
            **kwargs
        )

    def setup_full(self, model):
        res = super(Property, self).setup_full(model)
        if self.property_onsetup:
            self.property_onsetup(self, model)
        return res

    def setter(self, f):
        return self.new(setter=f)

    def deleter(self, f):
        return self.new(deleter=f)

    def onsetup(self, f):
        return self.new(onsetup=f)

    def __get__(self, instance, owner):
        if instance is None:
            return self
        else:
            if not instance:
                return self.null(instance.env)
            instance.ensure_one()
            return self.property_getter(instance)

    def __set__(self, instance, value):
        if self.property_setter:
            instance.ensure_one()
            self.property_setter(instance, value)
        else:
            raise TypeError('Setting to read-only Property')

    def __delete__(self, instance):
        if self.property_deleter:
            instance.ensure_one()
            self.property_deleter(instance)
        else:
            raise TypeError('Deleting undeletable Property')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from xoeuf import MAJOR_ODOO_VERSION
from xoeuf.models.proxy import ResUsers as Users
from xoeuf.odoo import SUPERUSER_ID
from xoeuf.odoo.tests.common import TransactionCase, HttpCase
from xoeuf.odoo.tests.common import at_install, post_install


class TestModelProxy(TransactionCase):
    def test_must_the_right_self(self):
        with self.assertRaises(RuntimeError):
            Users.search([])

    def test_finds_self(this):
        self = this.env['ir.model']  # noqa: make self a valid recordset
        Users.create({'name': 'John Doe', 'login': 'john.doe'})
        this.assertEqual(Users.search([]), self.env['res.users'].search([]))

    def test_instances_check(this):
        self = this.env['ir.model']  # noqa: make self a valid recordset
        root = Users.browse(SUPERUSER_ID)
        this.assertIn(root, Users._instances_)
        this.assertNotIn(self, Users._instances_)
        this.assertNotIn(object(), Users._instances_)


@at_install(False)
@post_install(True)
class TestHTTPModelProxy(HttpCase):
    @staticmethod
    def getcode(response):
        if MAJOR_ODOO_VERSION < 11:
            return response.getcode()
        else:
            # Odoo 11+ uses requests to load the URL.
            return response.status_code

    @unittest.skipIf(MAJOR_ODOO_VERSION >= 9,
                     'Does not raises the error in Odoo 9+')
    def test_request_no_auth(self):
        # The controller won't be able to find a proper environment and fail
        # with an AttributeError, we'll see as an error 500
        self.authenticate('admin', 'admin')
        response = self.url_open('/test_proxy_none')
        code = self.getcode(response)
        self.assertEqual(code, HTTP_SERVER_ERROR)

    def test_request_with_auth(self):
        response = self.url_open('/test_proxy_pub')
        code = self.getcode(response)
        self.assertEqual(code, HTTP_OK)


HTTP_OK = 200
HTTP_SERVER_ERROR = 500



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ----------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

from odoo.tests.common import TransactionCase

Hours = {
    '15:30': 15.5,
    '12:00': 12,
    '9:45': 9.75
}


class TestTimeRange(TransactionCase):
    def setUp(self):
        super(TestTimeRange, self).setUp()
        self.Model = self.env['test.time.range']

    def test_compute_range(self):
        obj = self.Model.create(dict(
            time_value=Hours['15:30']
        ))

        self.assertEqual(obj.range_value, 'afternoon')

        obj.time_value = Hours['9:45']

        self.assertEqual(obj.range_value, 'morning')

        obj.time_value = Hours['12:00']

        self.assertEqual(obj.range_value, 'noon')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

from xoeuf.odoo.tests.common import TransactionCase
from xoeuf.models.extensions import get_ref


class TestModels(TransactionCase):
    def test_get_ref(self):
        Foo = self.env['test_xoeuf_models.foobar']
        record = self.env.ref('test_models.record1')
        self.assertEqual(
            record,
            get_ref(Foo, 'record1'),
        )
        self.assertEqual(
            record,
            Foo.get_ref('record1'),
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

from collections import namedtuple
from xoutil.eight import string_types

from xoeuf import models, api

__all__ = ['Enumeration']

Member = namedtuple('Member', ('name', 'value'))


def Enumeration(enumclass, *args, **kwargs):
    '''Create an enumeration field.

    The `enumclass` argument must be either an `enumeration class` or a
    fully qualified name of an enumeration class.

    The column in the DB will be either of type INTEGER or a CHAR: If **all**
    values of the enumeration are integers (instances of `int`, possibly by
    subclassing), the DB column will be a integer unless you pass a keyword
    argument 'force_char_column' set to True.  Otherwise, it will be a char
    with the *name* of the enumeration's key.

    .. warning:: In a future release we might drop the INTEGER/CHAR
       variation.

       This is to avoid possible mistakes: you create an enumeration with
       integers and afterwards you change to other types; that would require a
       migration step in the DB to convert old integer values to names.

    .. note:: Even in Python 2, we test for instances of `int`.  So values of
       type `long` won't be considered integers.

    Enumeration classes are required to:

    - have an attribute ``__members__`` with a `collections.Mapping`:class:
      from names to values.

    - have an attribute for each name in ``__member__`` with the same value as
      in the mapping.

    Those rules imply::

       >>> all(getattr(enumclass, name) is value
       ...     for name, value in enumclass.__members__.items())

    These requirements are compatible with Python 3.4's `enum.Enum`:class:
    However, we don't require that values are instances of the enumeration
    class.

    .. versionchanged:: 0.36.0 A new generalized enumeration field.  Maintains
       DB compatibility (does not need migrations), but do require changes in
       the code.

    .. warning:: To avoid mistakes in the final 1.0 release (or some releases
       before that) the column DB will always be a CHAR representing the
       member's name.

    .. versionchanged:: 0.47.0 Add keyword parameter 'force_char_column'.

    '''
    from xoeuf import fields
    from xoutil.objects import import_object, classproperty

    force_char_column = kwargs.get('force_char_column', False)
    enumclass = import_object(enumclass)
    if force_char_column:
        members_integers = False
    else:
        members_integers = all(
            isinstance(value, int)
            for value in enumclass.__members__.values()
        )
    Base = fields.Integer if members_integers else fields.Char

    class EnumeratedField(Base, _EnumeratedField):
        @classproperty
        def members(cls):
            return enumclass.__members__

        def setup_full(self, model):
            # Injects a new class in the model MRO, so that we can guarantee
            # the expectation about the methods create, write and search.
            #
            # The alternative would be to implement pre_save and pre_search
            # receivers.  But that would run for ALL models regardless of the
            # presence of Enumeration fields.  Also, we need to rewrite the
            # argument in the 'search' method, and signals are designed for
            # such use case (although they work at the moment).
            #
            cls = type(model)
            if EnumerationAdapter not in cls.__bases__:
                cls.__bases__ = (EnumerationAdapter, ) + cls.__bases__
            return super(EnumeratedField, self).setup_full(model)

        def convert_to_read(self, value, record, use_name_get=True):
            if value is not None and value is not False:
                return self.get_member_by_value(value).value
            else:
                return value

        if Base is fields.Integer:
            def convert_to_write(self, value, record):
                if value:
                    return self.get_member_by_value(value).value
                else:
                    return value

            def convert_to_cache(self, value, record, validate=True):
                if value:
                    return self.get_member_by_value(value).value
                elif value == 0:
                    # Odoo converts NULL to 0 for Integers, we try to find a
                    # 0-member, and return False if not found.
                    try:
                        return self.get_member_by_value(value).value
                    except ValueError:
                        return False
                return value

        else:
            def convert_to_write(self, value, record):
                if value is not None and value is not False:
                    if value in enumclass.__members__.values():
                        member = self.get_member_by_value(value)
                        # Our EnumerationAdapter takes care of doing the right
                        # thing when writing to the DB, also convert_to_column
                        # does.
                        return member.value
                return value

            def convert_to_cache(self, value, record, validate=True):
                if value not in enumclass.__members__.values():
                    if value is not None and value is not False:
                        return self.get_member_by_name(value).value
                return value

            # NOTE: The parameter `values` was introduced in Odoo 11; the
            # parameter validate was introduced in Odoo 12.  We don't use
            # either; but declare them both to work across the three Odoo
            # versions.
            def convert_to_column(self, value, record, values=None,
                                  validate=True):
                if value in enumclass.__members__.values():
                    return Base.convert_to_column(
                        self,
                        self.get_member_by_value(value).name,
                        record
                    )
                else:
                    return Base.convert_to_column(self, value, record)

    return EnumeratedField(*args, **kwargs)


class EnumerationAdapter(object):
    'Adapt the create/write/search method to Enumeration fields.'
    # See the note in setup_full above.

    # Odoo 11 requires all bases to have these attributes.  See
    # odoo/models.py, method _build_model_attributes in lines 515-555 (at the
    # time of writing).
    _table = None
    _description = None
    _sequence = None
    _inherits = _depends = {}
    _sql_constraints = []
    _constraints = []

    # Odoo 12 requires this attribute.  See odoo/models.py, line 528
    _inherit = None

    @api.model
    @api.returns(*models.BaseModel.search._returns)
    def search(self, args, *pos_args, **kwargs):
        for index, query_part in enumerate(args):
            if not isinstance(query_part, string_types):
                fieldname, operator, operands = query_part
                field = self._fields.get(fieldname, None)
                if isinstance(field, _EnumeratedField):
                    if operator in ('=', '!='):
                        values = _get_db_value(field, operands)
                    elif operator in ('in', 'not in'):
                        values = [_get_db_value(field, o) for o in operands]
                    else:
                        raise TypeError(
                            'Unsupported operator %r for an enumeration field' % operator
                        )
                    args[index] = (fieldname, operator, values)
        return super(EnumerationAdapter, self).search(args, *pos_args, **kwargs)

    @api.model
    @api.returns('self', lambda value: value.id)
    def create(self, values):
        for fieldname, value in dict(values).items():
            field = self._fields.get(fieldname, None)
            values[fieldname] = _get_db_value(field, value)
        return super(EnumerationAdapter, self).create(values)

    @api.multi
    def write(self, values):
        for fieldname, value in dict(values).items():
            field = self._fields.get(fieldname, None)
            values[fieldname] = _get_db_value(field, value)
        return super(EnumerationAdapter, self).write(values)


class _EnumeratedField(object):
    @classmethod
    def get_member_by_value(cls, value):
        '''Find the enumclass's member that is equal to `value`.'''
        try:
            return next(
                Member(k, v)
                for k, v in cls.members.items()
                if v == value
            )
        except StopIteration:
            raise ValueError(
                'Invalid member %r of enumeration %r' % (value, cls.members)
            )

    @classmethod
    def get_member_by_name(cls, name):
        '''Find the enumclass's member by name'''
        try:
            return Member(name, cls.members[name])
        except (AttributeError, KeyError):
            raise ValueError(
                'Invalid key %r of enumeration %r' % (name, cls.members)
            )


def _get_db_value(field, value):
    from xoeuf.odoo import fields
    if value is None or value is False:  # and not field.required
        return value
    if isinstance(field, _EnumeratedField):
        member = field.get_member_by_value(value)
        if isinstance(field, fields.Integer):
            return member.value
        else:
            return member.name
    else:
        return value



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#
from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

import unittest
from hypothesis import strategies, given
from datetime import datetime

formats = strategies.sampled_from(
    ('%H:%M', '%H:%M:%S', '%H:%M:%S.%f')
)


class TestTimeRange(unittest.TestCase):
    @given(strategies.times(), strategies.times())
    def test_timerange(self, t1, t2):
        from xoeuf.fields.timerange.utils import TimeRange
        start = min(t1, t2)
        end = max(t1, t2)
        timerange = TimeRange(start, end)
        self.assertEqual(eval(repr(timerange)), timerange)
        if start != end:
            with self.assertRaises(ValueError):
                TimeRange(end, start)

    @given(strategies.times(), strategies.times(), formats)
    def test_timerange_fmt(self, t1, t2, fmt):
        from xoeuf.fields.timerange.utils import TimeRange
        # Since any of t1 or t2 will be formatted, for the 'end', I have to make
        # sure that start < end.
        t1 = datetime.strptime(t1.strftime(fmt), fmt).time()
        t2 = datetime.strptime(t2.strftime(fmt), fmt).time()
        start = min(t1, t2)
        end = max(t1, t2)
        TimeRange(start, end.strftime(fmt))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

# TODO: @manu, At least in Odoo 10, there is 'openerp.sentrylog'
try:
    from openerp.sentrylog import *   # noqa: reexport
except ImportError:
    from ._sentrylog import *  # noqa: reexport



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

'''Xœuf are basic services for OpenERP and Open Object.

The name is composed by:
  * x: The starting letter for almost all Merchise projects.
  * oe: Open and ERP initials.
  * œuf: Is "egg" in french.

'''
from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)


from . import modules  # noqa; bootstrap 'xoeuf.odoo'
from . import signals  # noqa; bootstrap signals
from .osv import orm   # bootstrap 'orm' (injects _RELATED in XMLs 'eval')

from xoeuf.odoo import SUPERUSER_ID  # noqa
from xoeuf.odoo.release import version_info as ODOO_VERSION_INFO  # noqa

MAJOR_ODOO_VERSION = ODOO_VERSION_INFO[0]  # noqa

del orm



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

from enum import IntEnum, Enum
from xoeuf import models, fields, api


class COLORS(IntEnum):
    Blue = 0
    Red = 1
    Green = 2


class CARS(Enum):
    FORD = object()
    CHEV = object()


class Pax(object):
    single = 1
    double = 2
    __members__ = {'single': single, 'double': double}


class WorkType(int):
    pass


class WORK_TYPE(object):
    easy = WorkType(0)
    hard = WorkType(1)
    __members__ = {'easy': easy, 'hard': hard}


class Model(models.Model):
    _name = 'test.enum.model'
    color = fields.Enumeration(COLORS)
    color_name = fields.Selection(
        selection=[(name, name) for name in COLORS.__members__],
        compute='_compute_color_name',
        inverse='_set_color_name',
        store=False,
    )

    car = fields.Enumeration(CARS)
    pax = fields.Enumeration(Pax)
    wtype = fields.Enumeration(WORK_TYPE, default=WORK_TYPE.hard)

    color2 = fields.Enumeration(COLORS, force_char_column=True)
    color2_name = fields.Selection(
        selection=[(name, name) for name in COLORS.__members__],
        compute='_compute_color2_name',
        inverse='_set_color2_name',
        store=False,
    )

    color3 = fields.Enumeration(COLORS, force_char_column=True,
                                default=COLORS.Red)

    @api.multi
    @api.depends('color')
    def _compute_color_name(self):
        for record in self:
            record.color_name = next(
                name
                for name, value in COLORS.__members__.items()
                if value == record.color
            )

    @api.multi
    def _set_color_name(self):
        for record in self:
            record.color = COLORS.__members__[record.color_name]

    @api.multi
    @api.depends('color2')
    def _compute_color2_name(self):
        for record in self:
            record.color2_name = next(
                name
                for name, value in COLORS.__members__.items()
                if value == record.color2
            )

    @api.multi
    def _set_color2_name(self):
        for record in self:
            record.color2 = COLORS.__members__[record.color2_name]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#
from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

from xoeuf import api, fields, models
from odoo import exceptions

TEST_MODEL_NAME = 'test_view_model.model'
TEST_MODEL_NAME2 = 'test_view_model.model2'


class ExampleMixin(models.AbstractModel):
    _name = 'example.mixin'

    is_example_mixin = True


class Model(models.Model):
    _name = TEST_MODEL_NAME

    name = fields.Char()
    value = fields.Integer()


ViewModel1 = models.ViewModel(
    'view_model1',
    TEST_MODEL_NAME,
    mixins=['example.mixin']
)


class ViewModel2(models.ViewModel('view_model2', TEST_MODEL_NAME)):

    @api.constrains('value')
    def check_value(self):
        for record in self:
            if record.value < 0:
                raise exceptions.ValidationError('value must be positive.')


class Model2(models.Model):
    _name = TEST_MODEL_NAME2

    ref_model = fields.Many2one(TEST_MODEL_NAME)
    model_value = fields.Integer(compute='get_model_value', store=True)

    @api.depends('ref_model', 'ref_model.value')
    def get_model_value(self):
        for record in self:
            record.model_value = record.ref_model.value



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

'''Odoo API Extensons.

'''

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

from xoutil.decorator.meta import decorator as _xdecorator

from xoeuf.odoo import api as _odoo_api


# TODO: `copy_members` will be deprecated in xoutil 1.8, use instead the same
# mechanisms as `xoutil.future`.
from xoutil.modules import copy_members as _copy_python_module_members
this = _copy_python_module_members(_odoo_api)
del _copy_python_module_members


def contextual(func):
    '''Decorate a function to run within a proper Odoo environment.

    You should decorate every function that represents an "entry point" for
    working with the ORM.  A proper `Environment`:class: is entered upon
    calling the function.

    '''
    def inner(*args, **kwargs):
        with _odoo_api.Environment.manage():
            return func(*args, **kwargs)
    return inner


def requires_singleton(f):
    '''An idiomatic alias for `api.multi()`.

    This is exactly the same as `api.multi()`, however it's expected to be
    used when the code you're decorating requires a singleton recordset.

    Notice we don't fail at the method call, but only if the actual code
    executes a command that requires such a condition to be met (for instance,
    accessing a field in ``self``.)

    '''
    return _odoo_api.multi(f)


def mimic(original):
    '''Apply the API guess of `original` to the decorated function.

    Usage::

       def f1(self, cr, uid, ids, context=None):
           # Actually any valid signature

       @api.mimic(f1)
       def f2(*args, **kwargs):
           pass

    '''
    import types
    method = _odoo_api.guess(original)
    # Odoo stores the decorator in the _api attribute.  But Odoo 10 only
    # stores the name of the API method.
    decorator = method._api
    if isinstance(decorator, types.FunctionType):
        return decorator
    else:
        return getattr(_odoo_api, decorator)


@_xdecorator
def from_active_ids(f, leak_context=False):
    '''Decorator that ensures `self` comes from 'active_ids' in the context.

    The context key 'active_model' must be set and match the recordset's
    model.  If the 'active_model' key does not match the recordset's model,
    call `f` with the given recordset, i.e act like `api.multi`:func:.

    If 'active_model' matches the recordset's, and 'active_ids' is not empty,
    run `f` with the recordset of active ids.

    `f` is automatically decorated with `api.multi`:func:.

    The expected use is in methods from a server action linked to an
    ir.value.  In those cases `self` is normally the first selected record,
    but you want it to be run with all selected records.

    If `leak_context` is False (the default), calls to other methods decorated
    with `from_active_ids`:func: won't take the ids from the 'active_ids'.

    .. versionchanged:: 0.34.0 Add the `leak_context` argument.  And allow `f`
       to take arguments (other than `self`).

    .. versionchanged:: 0.35.0 The `leak_context` defaults to False, and it
       does not change the 'active_model' key in the context.

    '''
    from functools import wraps
    from xoutil.context import Context

    @multi  # noqa
    @wraps(f)
    def inner(self, *args, **kwargs):
        model = self._name
        if _SKIP_ACTIVE_IDS in Context:
            this = self
        else:
            active_model = self.env.context.get('active_model')
            if active_model == model:
                active_ids = self.env.context.get('active_ids', ())
                if active_ids:
                    this = self.browse(active_ids)
                else:
                    this = self
            else:
                this = self
        with leaking_context(leak_context):
            return f(this, *args, **kwargs)

    return inner


_SKIP_ACTIVE_IDS = object()
_DONT_SKIP_ACTIVE_IDS = object()


def leaking_context(leak=False):
    from xoutil.context import Context

    if not leak:
        return Context(_SKIP_ACTIVE_IDS)
    else:
        return Context(_DONT_SKIP_ACTIVE_IDS)


def onupdate(*args):
    """A decorator to trigger updates on dependencies.

    Return a decorator that specifies the field dependencies of an updater
    method.  Each argument must be a string that consists in a dot-separated
    sequence of field names::

        @api.onupdate('partner_id.name', 'partner_id.is_company')
        def update_pname(self):
            for record in self:
                if record.partner_id.is_company:
                    record.pname = (record.partner_id.name or "").upper()
                else:
                    record.pname = record.partner_id.name

    One may also pass a single function as argument. In that case, the
    dependencies are given by calling the function with the field's model.

    .. note:: ``@onupdate`` is very similar to ``@constraint`` but with just
       one key difference: It allow dot-separated fields in arguments.

    .. versionadded: 0.46.0

    """
    if args and callable(args[0]):
        args = args[0]
    elif any('id' in arg.split('.') for arg in args):
        raise NotImplementedError(
            "Updater method cannot depend on field 'id'."
        )
    return _odoo_api.attrsetter('_onupdates', args)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

from xoutil.future.codecs import safe_decode

from xoeuf.odoo.tests.common import TransactionCase, at_install, post_install
from xoeuf.signals import (
    mock_replace,
    post_create,
    pre_create,
    write_wrapper,
    pre_fields_view_get,
)
from xoeuf.odoo.addons.test_signals.models import (
    post_save_receiver,
    post_save_receiver_all_models,
    pre_save_receiver,
    wrap_nothing,
    pre_fvg_receiver,
)


@at_install(False)
@post_install(True)
class TestXoeufSignals(TransactionCase):
    def setUp(self):
        super(TestXoeufSignals, self).setUp()
        self.Model = self.env['test_signals.signaling_model']
        self._ready_before = self.env.registry.ready
        self.env.registry.ready = True

    def tearDown(self):
        self.env.registry.ready = self._ready_before

    def test_post_create(self):
        with mock_replace(post_create, post_save_receiver) as mock_post_create, \
             mock_replace(pre_create, post_save_receiver) as mock_pre_create:
            self.Model.create(dict(name='My name'))
            self.assertTrue(mock_post_create.called)
            self.assertFalse(mock_pre_create.called)

    def test_pre_create(self):
        with mock_replace(post_create, pre_save_receiver) as mock_post_create, \
             mock_replace(pre_create, pre_save_receiver) as mock_pre_create:
            self.Model.create(dict(name='My name'))
            self.assertFalse(mock_post_create.called)
            self.assertTrue(mock_pre_create.called)

    def test_writer_wrap(self):
        def wrapper(sender, signal, *args, **kwargs):
            yield

        who = self.Model.create(dict(name='My name'))
        with mock_replace(write_wrapper, wrap_nothing, side_effect=wrapper) as mock:
            self.assertFalse(mock.called)
            who.write(dict(name='My new name'))
            self.assertTrue(mock.called)
        with mock_replace(write_wrapper, wrap_nothing, side_effect=wrapper) as mock:
            who.name = 'My new other name'
            self.assertTrue(mock.called)

    def test_not_called_for_partners(self):
        with mock_replace(post_create, post_save_receiver) as mock, \
             mock_replace(post_create, post_save_receiver_all_models) as mock_all:
            partner = self.env['res.partner'].create(dict(name='Contact Info'))
            partner.email = 'a@b.c'
            self.assertFalse(mock.called)
            self.assertTrue(mock_all.called)

    def test_fvg_in_abstract_models(self):
        who = self.Model.create(dict(name='My name'))
        with mock_replace(pre_fields_view_get, pre_fvg_receiver) as mock:
            result = who.fields_view_get()
            self.assertTrue(mock.called)
            self.assertIn('fgv-is-present', safe_decode(result['arch'], 'utf-8'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#
from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

from psycopg2 import IntegrityError
from xoeuf.odoo.tests.common import TransactionCase


class TestO2O(TransactionCase):
    def test_one2one_injects_restriction(self):
        C = self.env['test.one2one.c']
        c = C.create({'name': 'I am a C'})
        a = c.a_id
        with self.assertRaises(IntegrityError):
            C.create({'name': 'I am trying to steal the A', 'a_id': a.id})

    def test_one2one_baseline(self):
        B = self.env['test.one2one.b']
        b = B.create({'name': 'I am a B'})
        a = b.a_id
        B.create({'name': 'I can steal the A', 'a_id': a.id})

    def test_one2one_cascade(self):
        C = self.env['test.one2one.c']
        c = C.create({'name': 'I am a C'})
        a = c.a_id
        a.unlink()
        self.assertFalse(c.exists())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)


DEFAULT_COMMAND = str('server')

# Makes sure modules are patched before any command-related code is
# invoked.
from xoeuf.modules import patch_modules
patch_modules()


from xoutil.eight.meta import metaclass
from xoutil.cli import Command as BaseCommand


class CommandsProxy(BaseCommand):
    '''Define a proxy to register all OpenERP CLI commands to "xoutil.cli".

    '''
    __new__ = None  # Can't be instantiated

    @classmethod
    def __commands__(cls):
        '''Get all OpenERP registered commands.

        Discard add-ons that raise errors when importing them.

        '''
        name = '__commands_cache__'
        res = getattr(cls, name, None)
        if not res:
            try:
                # TODO: which version is that? do we support it?
                from openerp.cli import commands
            except ImportError:
                from xoeuf.odoo.cli.command import commands
            from xoeuf.odoo.modules.module import initialize_sys_path

            cls._discover_addons_path()
            initialize_sys_path()
            res = commands
            setattr(cls, name, res)
        return res.values()

    @staticmethod
    def _discover_addons_path():
        import sys
        args = sys.argv[1:]
        prefix = '--addons-path='
        addons_path = next((a for a in args if a.startswith(prefix)), None)
        if addons_path:
            from xoeuf.odoo.tools import config
            config.parse_config([addons_path])


BaseCommand.set_default_command(DEFAULT_COMMAND)


class CommandType(type(BaseCommand)):
    def __new__(cls, name, bases, attrs):
        from xoeuf.api import contextual
        run = attrs.get('run', None)
        if run:
            attrs['run'] = contextual(run)
        return super(CommandType, cls).__new__(cls, name, bases, attrs)


class Command(metaclass(CommandType), BaseCommand):
    @staticmethod
    def invalidate_logging(base=None):
        '''Force the logger `base` to report only CRITICAL messages.

        :param base: The name of logger to invalidate.  None is the root
                     logger *and* the "openerp" logger.

        '''
        import logging
        logger = logging.getLogger(base)
        logger.setLevel(logging.CRITICAL)
        if not base:
            Command.invalidate_logging('openerp')


del BaseCommand


# TODO: Loader?
from . import migration
from . import secure as _secure
from . import addons as _addons
del _secure, _addons, migration



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

'''Extends/Overrides the OpenERP's logging system to Sentry-based approach.

Sentry_ aggregates logs and lets you inspect the server's health by a web
application.

To configure, simply set the global `conf`:obj: dictionary and call
`patch_logging`:func:.

'''

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

import raven
from raven.transport.http import HTTPTransport
from raven.transport.threaded import ThreadedHTTPTransport
from raven.transport.gevent import GeventedHTTPTransport

from raven.utils.wsgi import get_headers, get_environ

try:
    import urlparse as _urlparse
except ImportError:
    import urllib.parse as _urlparse


# This module is about logging-only, not wrapping the WSGI application in a
# middleware, etc.

from xoutil.objects import setdefaultattr

# A dictionary holding the Raven's client keyword arguments.  You should
# modify this dictionary before patching the logging.
conf = {}

# The name of the context to the logger to avoid logging sentry-related
# errors.
SENTRYLOGGER = object()


# A singleton
_sentry_client = None


def get_client():
    from xoeuf.odoo.tools import config
    global _sentry_client
    overrides = config.misc.get('sentry', {})
    conf.update(overrides)
    if not _sentry_client and 'dsn' in conf:
        releasetag = conf.pop('release-tag', '')
        if 'release' not in conf:
            from xoeuf.odoo.release import version
            conf['release'] = '%s/%s' % (version, releasetag)
        transport = conf.get('transport', None)
        if transport == 'sync':
            transport = HTTPTransport
        elif transport == 'gevent':
            transport = GeventedHTTPTransport
        else:
            transport = ThreadedHTTPTransport
        conf['transport'] = transport
        _sentry_client = raven.Client(**conf)
    return _sentry_client


def patch_logging(override=True, force=False):
    '''Patch openerp's logging.

    :param override: If True suppress all normal logging.  All logs will be
           sent to the Sentry instead of being logged to the console.  If
           False, extends the loogers to sent the errors to the Sentry but
           keep the console log as well.

    :param force: Force the patching even if working with an Odoo
           implementation that supports Sentry.  This is basically useful for
           scripts like `mailgate` that run custom code but should log the
           same as the core of Odoo.

           If set to True, `override` will happen has well.

    The Sentry will only receive the error-level messages.

    '''
    try:
        from xoeuf.odoo import sentrylog  # noqa
    except ImportError:
        sentrylog = None
    if sentrylog and not force:
        return   # Bail out
    elif sentrylog:
        override = True

    import logging
    from raven.handlers.logging import SentryHandler as Base
    from xoeuf.odoo.netsvc import init_logger
    init_logger()

    def _require_httprequest(func):
        def inner(self, record):
            try:
                from xoeuf.odoo.http import request
                httprequest = getattr(request, 'httprequest', None)
                if httprequest:
                    return func(self, record, httprequest)
            except ImportError:
                # Not inside an HTTP request
                pass
            except RuntimeError:
                # When upgrading a DB the request may exists but the bound to
                # it does not.
                pass
        return inner

    class SentryHandler(Base):
        def _emit(self, record, **kwargs):
            self.set_record_tags(record)
            request_context = self._get_http_context(record)
            if request_context:
                self.client.http_context(request_context)
            user_context = self._get_user_context(record)
            if user_context:
                self.client.user_context(user_context)
            try:
                super(SentryHandler, self)._emit(record, **kwargs)
            except:  # noqa
                # We should never fail if emitting the log to Sentry fails.
                # Neither we should print the error, other programs may think
                # we have fail because of it: For instance, the mailgate
                # integrated with postfix does.
                pass
            finally:
                self.client.context.clear()

        @_require_httprequest
        def _get_http_context(self, record, request):
            urlparts = _urlparse.urlsplit(request.url)
            return {
                'url': '%s://%s%s' % (urlparts.scheme, urlparts.netloc,
                                      urlparts.path),
                'query_string': urlparts.query,
                'method': request.method,
                'data': self._get_http_request_data(request),
                'headers': dict(get_headers(request.environ)),
                'env': dict(get_environ(request.environ)),
            }

        @_require_httprequest
        def _get_user_context(self, record, request):
            return {
                'id': getattr(request, 'session', {}).get('login', None)
            }

        def _handle_cli_tags(self, record):
            import sys
            from itertools import takewhile
            tags = setdefaultattr(record, 'tags', {})
            if sys.argv:
                cmd = ' '.join(
                    takewhile(lambda arg: not arg.startswith('-'),
                              sys.argv)
                )
            else:
                cmd = None
            if cmd:
                import os
                cmd = os.path.basename(cmd)
            if cmd:
                tags['cmd'] = cmd

        @_require_httprequest
        def _handle_browser_tags(self, record, request):
            tags = setdefaultattr(record, 'tags', {})
            ua = request.user_agent
            if ua:
                tags['os'] = ua.platform.capitalize()
                browser = str(ua.browser).capitalize() + ' ' + str(ua.version)
                tags['browser'] = browser

        @_require_httprequest
        def _handle_db_tags(self, record, request):
            db = getattr(request, 'session', {}).get('db', None)
            if db:
                tags = setdefaultattr(record, 'tags', {})
                tags['db'] = db

        def _handle_fingerprint(self, record):
            from xoutil.names import nameof
            exc_info = record.exc_info
            if exc_info:
                _type, value, _tb = exc_info
                exc = nameof(_type, inner=True, full=True)
                if exc.startswith('psycopg2.'):
                    fingerprint = [exc]
                else:
                    fingerprint = getattr(value, '_sentry_fingerprint', None)
                if fingerprint:
                    if not isinstance(fingerprint, list):
                        fingerprint = [fingerprint]
                    record.fingerprint = fingerprint

        def _get_http_request_data(self, request):
            from xoeuf.odoo.http import JsonRequest, HttpRequest
            from xoeuf.odoo.http import request  # Let it raise
            # We can't simply use `isinstance` cause request is actual a
            # 'werkzeug.local.LocalProxy' instance.
            if request._request_type == JsonRequest._request_type:
                return request.jsonrequest
            elif request._request_type == HttpRequest._request_type:
                return request.params
            else:
                return None

        def can_record(self, record):
            res = super(SentryHandler, self).can_record(record)
            if not res:
                return False
            exc_info = record.exc_info
            if not exc_info:
                return res
            from xoeuf.odoo.exceptions import Warning
            ignored = (Warning, )
            try:
                from xoeuf.odoo.exceptions import RedirectWarning
                ignored += (RedirectWarning, )
            except ImportError:
                pass
            from xoeuf.odoo.exceptions import except_orm
            ignored += (except_orm, )
            _type, value, _tb = exc_info
            return not isinstance(value, ignored)

        def set_record_tags(self, record):
            methods = (
                getattr(self, m)
                for m in dir(self) if m.startswith('_handle_')
            )
            for method in methods:
                method(record)

    client = get_client()
    if not client:
        return

    level = conf.get('report_level', 'ERROR')

    def sethandler(logger, override=override, level=level):
        handler = SentryHandler(client=client)
        handler.setLevel(getattr(logging, level.upper(), logging.ERROR))
        if override or not logger.handlers:
            logger.handlers = [handler]
        else:
            logger.handlers.append(handler)

    for name in (None, 'openerp'):
        logger = logging.getLogger(name)
        sethandler(logger)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#
from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

import unittest


class TestXoeufImports(unittest.TestCase):
    def test_xoeuf_imports(self):
        'Xoeuf is importable in Python 2 and 3'
        from xoeuf import fields, models, api  # noqa
        try:
            from xoeuf.fields import Serialized    # noqa
        except ImportError:
            pass
        else:
            assert False, 'Serialized should not be exported'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#
from __future__ import division, print_function, absolute_import

from . import test_imports  # noqa
from . import test_localtime_as_remotetime  # noqa
from . import test_timerange_object  # noqa



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_imports)


def dev_tag_installed():
    import re
    import pkg_resources
    tag_start_regex = re.compile(r'[^\d\.]')
    try:
        dist = pkg_resources.get_distribution('xoeuf')
        version = dist.version
        match = tag_start_regex.search(version)
        if match:
            return version[match.start():]
        else:
            return None
    except Exception:
        return None


VERSION = '0.55.0'
RELEASE_TAG = dev_tag_installed() or ''



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#
from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

from xoeuf import api, models, fields, signals
from lxml import etree


class FVG(models.AbstractModel):
    _name = 'test_signals.fvg'

    @api.model
    def fields_view_get(self, view_id=None, view_type='form', toolbar=False,
                        submenu=False):
        result = super(FVG, self).fields_view_get(
            view_id=view_id,
            view_type=view_type,
            toolbar=toolbar,
            submenu=submenu
        )
        if view_type == 'form':
            doc = etree.XML(result['arch'])
            for node in doc.xpath('//field[@name="name"]'):
                node.set('fgv-is-present', 'yes')
            result['arch'] = etree.tostring(doc)
        return result


class SignalingModel(models.Model):
    _name = 'test_signals.signaling_model'
    _inherit = ['test_signals.fvg']

    name = fields.Char()


@signals.receiver(signals.post_save, sender='test_signals.signaling_model')
def post_save_receiver(sender, signal, **kwargs):
    pass


@signals.receiver(signals.post_save)
def post_save_receiver_all_models(sender, signal, **kwargs):
    pass


@signals.receiver(signals.pre_save, sender='test_signals.signaling_model')
def pre_save_receiver(sender, signal, **kwargs):
    pass


@signals.wrapper(signals.write_wrapper, sender='test_signals.signaling_model')
def wrap_nothing(sender, wrapping, *args, **kwargs):
    yield


@signals.receiver(signals.pre_search, sender='test_signals.signaling_model')
def pre_search_receiver(sender, signal, **kwargs):
    pass


@signals.receiver(signals.pre_fields_view_get,
                  sender='test_signals.signaling_model')
def pre_fvg_receiver(sender, signal, **kwargs):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#
from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

from xoeuf import fields, models


class Mixin(models.AbstractModel):
    _name = 'test_localized_dt.mixin'

    tzone = fields.Char(default='America/Havana')
    dt = fields.Datetime()
    dt_at_tzone = fields.LocalizedDatetime('dt', 'tzone')


class ModelA(models.Model):
    _name = 'test_localizated_dt.model'
    _inherit = ['test_localized_dt.mixin']


class ModelB(models.Model):
    _name = 'test_localized_dt.inherited'
    _inherits = {models.get_modelname(ModelA): 'a_id'}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#
from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

import unittest

from hypothesis import strategies, given
from hypothesis.extra.pytz import timezones
from xoeuf.tools import localtime_as_remotetime, normalize_datetime


class TestLocalAsRemote(unittest.TestCase):
    @given(
        strategies.datetimes(
            min_value=normalize_datetime('1912-04-14'),
            max_value=normalize_datetime('2100-12-31')
        ),
        timezones(),
        timezones()
    )
    def test_local_remote_time(self, _dt, from_tz, to_tz):
        _dt = normalize_datetime(_dt)
        self.assertEqual(
            _dt,
            localtime_as_remotetime(
                localtime_as_remotetime(_dt, from_tz, to_tz),
                to_tz,
                from_tz
            )
        )
        self.assertEqual(
            _dt,
            localtime_as_remotetime(
                localtime_as_remotetime(_dt, from_tz, to_tz, ignore_dst=True),
                to_tz,
                from_tz,
                ignore_dst=True
            )
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#
{
    'name': 'test_one2one',
    'author': 'Merchise Autrement [~º/~]',
    'installable': True,
    'auto_install': False,
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)


from xoeuf.odoo.fields import *          # noqa: reexport

from .datetime import LocalizedDatetime  # noqa: reexport
from .properties import Property         # noqa: reexport
from .monetary import Monetary           # noqa: reexport
from .timespan import TimeSpan           # noqa: reexport
from .enumeration import Enumeration     # noqa: reexport
from .timezone import TimezoneSelection  # noqa: reexport
from .timedelta import TimeDelta         # noqa: reexport
from .one2one import One2one             # noqa: reexport
from .timerange import TimeRange         # noqa: reexport


try:
    del Serialized  # noqa
except NameError:
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

from xoeuf import models, fields


class Node(models.Model):
    _name = 'test_xoeuf_orm.node'
    parent = fields.Many2one(_name)
    children = fields.One2many(_name, 'parent')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ----------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

from xoeuf import fields, models


class TestTimeRange(models.Model):
    _name = 'test.time.range'

    time_value = fields.Float()
    range_value = fields.TimeRange(
        time_field='time_value',
        selection=[
            ('morning', 'Morning', '6:00', '11:59'),
            ('noon', 'Noon', '12:00', '13:59'),
            ('afternoon', 'Afternoon', '14:00', '18:59')
        ]
    )
